package com.cqut.stack.bn.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.cqut.stack.bn.dao.MetadataMapper;
import com.cqut.stack.bn.pojo.dto.metadata.QueryMetaDataListDTO;
import com.cqut.stack.bn.pojo.dto.metadata.TableDTO;
import com.cqut.stack.bn.pojo.entity.DatabaseField;
import com.cqut.stack.bn.pojo.entity.DatabaseTable;
import com.cqut.stack.bn.pojo.vo.matadata.QueryMetaDataListVO;
import com.cqut.stack.bn.pojo.vo.matadata.TableInfo;
import com.cqut.stack.bn.service.DatabaseFieldService;
import com.cqut.stack.bn.service.DatabaseTableService;
import com.cqut.stack.bn.service.MetadataService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class MetadataServiceImpl implements MetadataService {

    @Resource
    private MetadataMapper metadataMapper;

    @Resource
    private DatabaseTableService databaseTableService;

    @Resource
    private DatabaseFieldService databaseFieldService;

    @Override
    public QueryMetaDataListVO queryMetaDataList(QueryMetaDataListDTO dto) {
        // 分页查询数据表idList
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        LambdaQueryWrapper<DatabaseTable> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(DatabaseTable::getId)
                .like(StringUtils.isNotEmpty(dto.getTableRemark()),DatabaseTable::getTableRemark,dto.getTableRemark())
                .eq(ObjectUtil.isNotEmpty(dto.getOwnerType()),DatabaseTable::getOwnerType,dto.getOwnerType());
        List<DatabaseTable> databaseTableList = databaseTableService.list(queryWrapper);
        PageInfo<DatabaseTable> pageInfo = new PageInfo<>(databaseTableList);
        List<Integer> tableIdList = databaseTableList.stream().map(DatabaseTable::getId).collect(Collectors.toList());
        dto.setTableIdList(tableIdList);

        // 查询每个数据表对应的数据字段
        List<TableInfo> tableInfoList = metadataMapper.queryMetaDataList(dto);
        QueryMetaDataListVO queryMetaDataListVO = new QueryMetaDataListVO();
        queryMetaDataListVO.setTableInfoList(tableInfoList);
        queryMetaDataListVO.setTotal(pageInfo.getTotal());
        return queryMetaDataListVO;
    }

    @Override
    public String generateTable(Integer tableId) {
        LambdaQueryWrapper<DatabaseTable> qwTable = new LambdaQueryWrapper<>();
        qwTable.eq(DatabaseTable::getId, tableId);
        DatabaseTable databaseTable = databaseTableService.getOne(qwTable);
        TableDTO table = BeanUtil.copyProperties(databaseTable, TableDTO.class);

        LambdaQueryWrapper<DatabaseField> qwField = new LambdaQueryWrapper<>();
        qwField.eq(DatabaseField::getTableId, tableId);

        table.setFields(databaseFieldService.list(qwField));

        return createSQL(table);
    }

        private String createSQL(TableDTO table){
        StringBuilder sb = new StringBuilder();
        sb.append("create table ");
        sb.append(table.getTableName());
        sb.append("(");
        sb.append("id int auto_increment, ");
        for (DatabaseField field : table.getFields()) {
            sb.append(field.getFieldName());
            sb.append(" ");
            if (field.getFieldType().equals("varchar")){
                sb.append("varchar(");
                sb.append(field.getFieldLength());
                sb.append(")");
            }else {
                sb.append(field.getFieldType());
            }
            sb.append(" ");
            if (field.getIsNull() == 0){
                sb.append("not null");
            }else {
                sb.append("null");
            }
            sb.append(" ");
            sb.append("comment");
            sb.append(" ");
            sb.append("'");
            sb.append(field.getRemark());
            sb.append("'");
            sb.append(",");
        }
        sb.append("primary key (id)");
        sb.append(")");
        sb.append(" ");
        sb.append("comment");
        sb.append(" ");
        sb.append("'");
        sb.append(table.getTableRemark());
        sb.append("'");
        sb.append(";");
        return sb.toString();
    }
}
